/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx;

import com.keyware.sonar.cxx.rules.SecurityDesignRuleRepository;
import com.keyware.sonar.cxx.rules.checkers.FlagLineRule;
import com.keyware.sonar.cxx.rules.checkers.SqlVarNameChecker;
import org.sonar.api.batch.fs.FilePredicates;
import org.sonar.api.batch.fs.InputFile;
import org.sonar.api.batch.rule.CheckFactory;
import org.sonar.api.batch.rule.Checks;
import org.sonar.api.batch.sensor.Sensor;
import org.sonar.api.batch.sensor.SensorContext;
import org.sonar.api.batch.sensor.SensorDescriptor;

public class FlagLineSensor implements Sensor {
    private final Checks<FlagLineRule> checks;


    public FlagLineSensor(CheckFactory checkFactory) {
        checks = checkFactory.create(SecurityDesignRuleRepository.REPOSITORY_SQL_KEY);
        checks.addAnnotatedChecks(SqlVarNameChecker.class);
    }

    @Override
    public void describe(SensorDescriptor descriptor) {
        //传感器名称
        descriptor.name("FlagLine1RuleSensor");
        //传感器识别的语言
        descriptor.onlyOnLanguages(CxxLanguage.KEY);
        //传感器扫描的规则库
        descriptor.createIssuesForRuleRepository(SecurityDesignRuleRepository.REPOSITORY_SQL_KEY);
    }

    @Override
    public void execute(SensorContext context) {
        FilePredicates p = context.fileSystem().predicates();
        for (InputFile inputFile : context.fileSystem().inputFiles(p.hasLanguages(CxxLanguage.KEY))) {
            checks.all().forEach(check -> {
                check.execute(context, inputFile, checks.ruleKey(check));
            });
        }
    }
}
