/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules;

import com.keyware.sonar.cxx.CxxLanguage;
import com.keyware.sonar.cxx.LogLanguage;
import com.keyware.sonar.cxx.rules.checkers.*;
import org.sonar.api.server.rule.RulesDefinition;
import org.sonar.cxx.squidbridge.annotations.AnnotationBasedRulesDefinition;

import java.util.Arrays;
import java.util.List;

/**
 * 信息安全性设计准则规则仓库定义实现类
 *
 * @author GuoXin
 * @date 2024/1/9
 */
public class SecurityDesignRuleRepository implements RulesDefinition {
    public final static String REPOSITORY_KEY = "cxx-security-design-rules";
    public final static String REPOSITORY_LOG_KEY = "log-security-design-rules";
    public final static String REPOSITORY_SQL_KEY = "sql-security-design-rules";
    public final static String REPOSITORY_NAME = "C++信息安全性设计准则";

    // 规则检查器的集合，当有新的规则开发完毕后，需要添加到下面的集合中
    public final static List<Class> RULE_CHECKERS = Arrays.asList(ABCVarNameChecker.class,BufferDataChecker.class, CmdDataVerifyChecker.class,
             DLLVerifyChecker.class,EncryptionAlgorithmChecker.class,ErrorMessageChecker.class,
            FileAccessChecker.class,FormatFunctionCheck.class,FVNRPassWordChecker.class,FVNRShaChecker.class,HighEncryptDesChecker.class,
            HostIdentityVerifyChecker.class,IntegerCountVerifyChecker.class,LogFileWriteChecker.class,NumericalCopyChecker.class,
            PassWordCountChecker.class,PathVerifyChecker.class,PRNGVerifyChecker.class,ReallocMainChecker.class,SendMessageChecker.class,
            SQLVerifyChecker.class,UserInputPasswordChecker.class,ValidatePasswordCheck.class,VerificationPathChecker.class,VirtualLockUsageChecker.class);


    @Override
    public void define(Context context) {
        var repository = context.createRepository(REPOSITORY_KEY, CxxLanguage.KEY).
                setName(REPOSITORY_NAME);
        new AnnotationBasedRulesDefinition(repository, CxxLanguage.KEY).addRuleClasses(false, RULE_CHECKERS);
        repository.done();

        var repository1 = context.createRepository(REPOSITORY_LOG_KEY, LogLanguage.KEY).
                setName(REPOSITORY_NAME);
        new AnnotationBasedRulesDefinition(repository1, LogLanguage.KEY).addRuleClasses(false, List.of(LogChecker.class));
        repository1.done();

        var repository2 = context.createRepository(REPOSITORY_SQL_KEY, CxxLanguage.KEY).
                setName(REPOSITORY_NAME);
        new AnnotationBasedRulesDefinition(repository2, CxxLanguage.KEY).addRuleClasses(false, List.of(SqlVarNameChecker.class));
        repository2.done();
    }
}
