/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：28所 C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules.checkers;

import com.sonar.cxx.sslr.api.AstNode;
import com.sonar.cxx.sslr.api.Grammar;
import org.sonar.check.Priority;
import org.sonar.check.Rule;
import org.sonar.cxx.parser.CxxGrammarImpl;
import org.sonar.cxx.squidbridge.annotations.ActivatedByDefault;
import org.sonar.cxx.squidbridge.annotations.SqaleConstantRemediation;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import java.util.ArrayList;
import java.util.List;

@Rule(key = "NumericalCopyChecker", name = "应返回该私有数值的副本", description = "当私有数组成员的数据需要作为公有函数的返回值时，应返回该私有数值的副本", priority = Priority.INFO, tags = {"28suo"})
@ActivatedByDefault
@SqaleConstantRemediation("5min")
public class NumericalCopyChecker extends SquidCheck<Grammar> {//当私有数组成员的数据需要作为公有函数的返回值时，应返回该私有数值的副本。

    public void init() {
        // 订阅要检查AST节点类型，用于在visitNode方法中检查该类型节点
        this.subscribeTo(
                CxxGrammarImpl.translationUnit
        );
    }

    public void visitNode(AstNode astNode) {
        List<AstNode> astNodeList = astNode.getDescendants(CxxGrammarImpl.declarator);
        List<String> lists = new ArrayList();
        for (AstNode desc : astNodeList) {
            if (desc.getTokenValue().contains("private")) {
                lists.add(desc.getTokenValue());
            }
        }
        for (AstNode as : astNodeList) {
            if (as.getTokenValue().contains("&") || as.getTokenValue().contains("*")) {
                List<AstNode> descendan = astNode.getDescendants(CxxGrammarImpl.statement);
                for (AstNode ast : descendan) {
                    List<AstNode> descendants = ast.getDescendants(CxxGrammarImpl.expression);
                    for (AstNode des : descendants) {
                        if ("return".equals(des.getParent().getTokenValue())) {
                            if (lists.contains(des.getTokenValue())) {
                                getContext().createLineViolation(this, "应返回该私有数值的副本", des);
                            }
                        }
                    }
                }
            }
        }
    }
}
