/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.java.ast.parser.ArgumentListTreeImpl;
import org.sonar.java.model.declaration.VariableTreeImpl;
import org.sonar.java.model.expression.IdentifierTreeImpl;
import org.sonar.java.model.expression.MethodInvocationTreeImpl;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.Collections;
import java.util.List;

/**
 * 应使用盐值计算口令
 *
 * @author RenFengJiang
 * @date 2024/1/11
 */
@Rule(key = "HashSaltPassWordChecker")
public class HashSaltPassWordChecker extends IssuableSubscriptionVisitor {

    @Override
    public List<Tree.Kind> nodesToVisit() {
        /**
         * Tree.Kind.METHOD：方法节点
         * Tree.Kind.BLOCK：方法的代码块节点
         * Tree.Kind.METHOD_INVOCATION： 方法的调用节点
         */
        return Collections.singletonList(Tree.Kind.METHOD);
    }

    @Override
    public void visitNode(Tree tree) {
        MethodTree node = (MethodTree) tree;
        var bodyVisitor = new MethodeBodyVisitor(this);
        node.block().accept(bodyVisitor);
    }


    static class MethodeBodyVisitor extends BaseTreeVisitor {
        //        存放盐值后的密码
        public String strPassWord = "";
        private final HashSaltPassWordChecker checker;

        MethodeBodyVisitor(HashSaltPassWordChecker checker) {
            this.checker = checker;
        }

        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
        //获取到方法调用
            ExpressionTree expressionTree = tree.methodSelect();
            if (expressionTree.is(Tree.Kind.MEMBER_SELECT)) {
                // 获取到调用的方法
                MemberSelectExpressionTree memberSelectExpressionTree = (MemberSelectExpressionTree) expressionTree;
                if ("setPassWord".equals(memberSelectExpressionTree.identifier().name())) {
                    Arguments arguments = tree.arguments();
                    for (ExpressionTree expressionTree1 : arguments) {
                        if (expressionTree1.is(Tree.Kind.IDENTIFIER)) {
                            IdentifierTree identifierTree = (IdentifierTree) expressionTree1;
                            // 判断set中使用的参数
                            if(!identifierTree.name().equals(strPassWord)){
                                checker.context.reportIssue(checker, identifierTree, "应使用盐值计算口令");
                            }
                        } else {
                            System.out.println(expressionTree1);
                            checker.context.reportIssue(checker, expressionTree1, "应使用盐值计算口令");
                        }
                    }
                } else if("BCrypt".equals(memberSelectExpressionTree.expression().toString()) && "hashpw".equals(memberSelectExpressionTree.identifier().name())){
                    Tree parent = memberSelectExpressionTree.parent();
                    if(parent instanceof MethodInvocationTreeImpl){
                        MethodInvocationTreeImpl methodInvocationTree = (MethodInvocationTreeImpl) parent;
                        Tree parent1 = methodInvocationTree.parent();
                        if(parent1 instanceof VariableTreeImpl){
                            VariableTreeImpl variableTree = (VariableTreeImpl) parent1;
//                            加盐后的参数名称
                            strPassWord = variableTree.simpleName().name();
                        }
                    }
                }
            }
        }
    }
}
