/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 使用sql语句前应对其进行验证
 *
 * @author RenFengJiang
 * @date 2024/1/14
 */
@Rule(key = "InputSQLVerifyChecker")
public class InputSQLVerifyChecker extends IssuableSubscriptionVisitor {

    @Override
    public List<Tree.Kind> nodesToVisit() {
        /**
         * Tree.Kind.METHOD：方法节点
         * Tree.Kind.BLOCK：方法的代码块节点
         * Tree.Kind.METHOD_INVOCATION： 方法的调用节点
         */
        return Collections.singletonList(Tree.Kind.METHOD);
    }

    @Override
    public void visitNode(Tree tree) {
        MethodTree blockTree = (MethodTree) tree;
        MethodeBodyVisitor bodyVisitor = new MethodeBodyVisitor(this);
        blockTree.accept(bodyVisitor);
    }

    static class MethodeBodyVisitor extends BaseTreeVisitor {
        private final InputSQLVerifyChecker checker;
        private List lists = new ArrayList<String>();
        MethodeBodyVisitor(InputSQLVerifyChecker checker){
            this.checker = checker;
        }

        @Override
        public void visitIfStatement(IfStatementTree tree) {
            ExpressionTree condition = tree.condition();
            condition.accept(new BaseTreeVisitor(){
                @Override
                public void visitIdentifier(IdentifierTree tree) {
                    lists.add(tree.name());
                }
            });
        }

        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
            ExpressionTree expressionTree = tree.methodSelect();
            if (expressionTree.is(Tree.Kind.MEMBER_SELECT)) {
                MemberSelectExpressionTree memberSelectExpressionTree = (MemberSelectExpressionTree) expressionTree;
                if ("prepareStatement".equals(memberSelectExpressionTree.identifier().name())) {
                    Arguments arguments = tree.arguments();
                    for (ExpressionTree argument : arguments) {
                        if (argument.is(Tree.Kind.IDENTIFIER)) {
                            IdentifierTree identifierTree = (IdentifierTree) argument;
                            String name = identifierTree.name();
                            // 判断使用的sql是否存在集合中
                            if (!lists.contains(name)) {
                                checker.context.reportIssue(checker, argument, "使用sql语句前应对其进行验证");
                            }
                        }
                    }
                }
            }
        }
    }
}
