/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.Collections;
import java.util.List;
import java.util.Locale;

/**
 * 应使用单向不可逆的加密算法
 *
 * @author RenFengJiang
 * @date 2024/1/13
 */
@Rule(key = "Md5PassWordVerifyChecker")
public class Md5PassWordVerifyChecker extends IssuableSubscriptionVisitor {
    @Override
    public List<Tree.Kind> nodesToVisit() {
        /**
         * Tree.Kind.METHOD：方法节点
         * Tree.Kind.BLOCK：方法的代码块节点
         * Tree.Kind.METHOD_INVOCATION： 方法的调用节点
         */
        return Collections.singletonList(Tree.Kind.METHOD);
    }

    @Override
    public void visitNode(Tree tree) {
        MethodTree node = (MethodTree) tree;
        var bodyVisitor = new MethodeBodyVisitor(this);
        node.block().accept(bodyVisitor);
    }


    static class MethodeBodyVisitor extends BaseTreeVisitor {
        //存放盐值后的密码
        public String strPassWord = "";
        private final Md5PassWordVerifyChecker checker;

        MethodeBodyVisitor(Md5PassWordVerifyChecker checker) {
            this.checker = checker;
        }

        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
            //获取到方法调用
            ExpressionTree expressionTree = tree.methodSelect();
            if (expressionTree.is(Tree.Kind.MEMBER_SELECT)) {
                MemberSelectExpressionTree memberSelectExpressionTree = (MemberSelectExpressionTree) expressionTree;
                if ("setPassWord".equals(memberSelectExpressionTree.identifier().name())) {
                    Arguments arguments = tree.arguments();
                    //改用arguments，这里没必要再用 instanceof 检查接口了
                    for (ExpressionTree argument : arguments) {
                        if (argument.is(Tree.Kind.IDENTIFIER)) {
                            IdentifierTree identifierTree = (IdentifierTree) argument;
                            String name = identifierTree.name();
                            if (!name.equals(strPassWord)) {
                                System.out.println(identifierTree);
                                checker.context.reportIssue(checker, identifierTree, "应使用单向不可逆的加密算法");
                            }
                        } else {
                            System.out.println(argument);
                            checker.context.reportIssue(checker, argument, "应使用单向不可逆的加密算法");
                        }
                    }
                } else if (memberSelectExpressionTree.identifier().name().toLowerCase(Locale.ROOT).contains("md5hex")
                        || memberSelectExpressionTree.identifier().name().toLowerCase(Locale.ROOT).contains("encrypt")) {
                    Tree parent = memberSelectExpressionTree.parent();
                    //对parent进行了instanceof检查，这是不正确的，应该使用is(Tree.Kind.SOMETHING)
                    if (parent.is(Tree.Kind.METHOD_INVOCATION)) {
                        MethodInvocationTree methodInvocationTree = (MethodInvocationTree) parent;
                        Tree parent1 = methodInvocationTree.parent();
                        //这同样是对parent1进行了instanceof检查，这是不正确的，应该使用is(Tree.Kind.SOMETHING)
                        if (parent1.is(Tree.Kind.VARIABLE)) {
                            VariableTree variableTree = (VariableTree) parent1;
                            strPassWord = variableTree.simpleName().name();
                        }
                    }
                }
            }
        }
    }
}
