/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */

package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.ModuleScannerContext;
import org.sonar.plugins.java.api.internal.EndOfAnalysis;
import org.sonar.plugins.java.api.tree.*;

import java.util.Collections;
import java.util.List;

/**
 * 应设置X-Frame-Options的值为deny
 * 1.获取所有方法调用节点
 * 2.判断方法中的参数是否否和要求
 *
 * @author RenFengJiang
 * @date 2024/1/22
 */
@Rule(key = "OptionsVerifyChecker")
public class OptionsVerifyChecker extends IssuableSubscriptionVisitor implements EndOfAnalysis {

    private boolean boo = true;

    @Override
    public List<Tree.Kind> nodesToVisit() {
        return Collections.singletonList(Tree.Kind.METHOD_INVOCATION);
    }

    @Override
    public void visitNode(Tree tree) {
        if(boo) {
            MethodInvocationTree methodInvocationTree = (MethodInvocationTree) tree;
            ExpressionTree expressionTree = methodInvocationTree.methodSelect();
            if (expressionTree.is(Tree.Kind.MEMBER_SELECT)) {
                MemberSelectExpressionTree selectExpressionTree = (MemberSelectExpressionTree) expressionTree;
                if("addHeader".equals(selectExpressionTree.identifier().name()) || "setHeader".equals(selectExpressionTree.identifier().name())) {
                    Arguments arguments = methodInvocationTree.arguments();
                    boolean one = false;
                    boolean two = false;
                    for (ExpressionTree argument : arguments) {
                        if(argument.is(Tree.Kind.STRING_LITERAL)){
                            String literalValue = ((LiteralTree) argument).value();
                            if ("\"X-Frame-Options\"".equals(literalValue)) {
                                one = true;
                            } else if("\"DENY\"".equals(literalValue)){
                                two = true;
                            }
                        }
                    }
                    if(one && two){
                        boo = false;
                    }
                }
            }
        }
    }

    @Override
    public void endOfAnalysis(ModuleScannerContext context) {
        if(boo){
            context.addIssueOnProject(this, "应设置X-Frame-Options的值为deny");
        }
    }
}
